#!/bin/bash
#
# Script to run Python 3 end-to-end tests on Ubuntu Jenkins instance.
#
# This script is deprecated and kept for backwards compatibility until
# the end-to-end tests have been reconfigured.
#
# This file is generated by l2tdevtools update-dependencies.py.

# Fail on error.
set -e

./config/linux/ubuntu_install_plaso.sh include-test;

./config/jenkins/linux/run_end_to_end_tests.sh $1;
